/*  Sql2File.java
 *  Copyright (c) 2001, Dennis Groenendijk
 *	Copyright (c) 2002-2007, Brains2B.org
 *  
 *  Created on 21 mei 2001, 2:32
 *  Created by: Dennis Groenendijk
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 /*  History
 *  25-05-2001 	Ratl 						Add default constructor
 *                  								Generate single name for Class
 *  19-12-2002 	dennis@brains2b.nl	Added writeHashCode functie
 *  23-01-2002 	dennis@brains2b.nl	Added setDate( java.util.Date)
 *  02-02-2003 	dennis@brains2b.nl	Fixed hashCode problem for float and double
  * 08-03-2003 		"			" 			Improved displaying info an logging
 *  03-11-2003 	dennis@brains2b.nl	Rewrote to use TypeWriter
 *  22-03-2004 	dennis@brains2b.nl 	Added option for optimistic lock
 * 												Fixed problem with equals methods and mutiple keys
 * 												Added javadoc, cleaned up a bit
 *  06-05-2004 	dennis@brains2b.nl	Fixed problem with different order of columns and primary key
 *  10-01-2005 	dennis@brains2b.nl   Added option for stripping values on a String
 *  13-02-2005 	dennis@brains2b.nl 	fixed problem with long in hashCode
 *  14-02-2005 	dennis@brains2b.nl	Do not create custom constructor for table with more than 255 columns
 *  20-07-2005 	dennis@brains2n.nl Do not add nano seconds to non timestamp dates
 *  03-10-2005 	dennis@brains2b.nl  changes for checks on reserved words 
 *  19-08-2006 	dennis@brains2b.nl	protect created methods against reserved methods
 *  26-09-2006	dennis@brains2b.nl	integrated HashCodeHelper, use OptimisticLock interface
 *  									Generate " " from accessors for strip quotes
 *  28-11-2006  dennis@brains2b.nl  #856 Use serialID instead of 0 for empty class
 *  30-01-2007  dennis@brains2b.nl  #909 Implemented using default values
 *  03-02-2007  dennis@brains2b.nl  #917 write L/d indicator on setting defaults
 *                                  #918 Use large columns for more than 200 columns
 *  04-02-2007  dennis@brains2b.nl  #923 set serialID in constructor.
 *  12-02-2007  dennis@brains2b.nl  serialID() should be used as function
 *  26-03-2007  dennis@brains2b.nl  fix for changes in SqlTableModel
 *  07-05-2007  dennis@brains2b.nl  Changes for new ColumnModel
 *  12-05-2007  dennis@brains2b.nl  Limit the number of columns in constructor to 126, otherwise Eclipse barfs
 *  14-05-2007  dennis@brains2b.nl  Detect keywords (sysdate supported)  in default value correctly
 *  15-05-2007  dennis@brains2b.nl  Don't use NULL or functions as default
 *  28-09-2007	dennis@brains2b.nl	Fixed writing defaults als Literal if needed
 *  17-01-2008	dennis@brains2b.nl  changes for new DataC version
 *  29-01-2008	dennis@brains2b.nl	generate code for lob's correctly 
 */
package org.brains2b.sql.toclass;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Modifier;
import java.util.Date;
import java.util.List;

import org.brains2b.log.Logger;
import org.brains2b.sql.meta.ColumnModel;
import org.brains2b.sql.meta.TableModel;
import org.brains2b.typewriter.ClassTemplate;
import org.brains2b.typewriter.CommentTemplate;
import org.brains2b.typewriter.ContentTemplate;
import org.brains2b.typewriter.IfTemplate;
import org.brains2b.typewriter.KeyWords;
import org.brains2b.typewriter.Literal;
import org.brains2b.typewriter.MethodTemplate;
import org.brains2b.util.JavaHelper;
import org.brains2b.util.StringHelper;

/** This class writes an SQL Table which is supplied through an <code>SqlTableModel</code>
 * to a data class, with the necessary accessors
 * <p>This class is called from <code>Sql2ClassConsole</code> and some other implementations
 * which supply the necessary information. If you use this class just to generate files look at 
 * <code>Sql2ClassConsole</code> first. 
 * <p>The naming of the new class file will be equal to the table name, but with initial
 *  caps. example: <i>EMPLOYEES will become Employees</i>. Underscores will be treated
 * as an instance to initialize caps again. example: <i>EMP_SALARIES will become EmpSalaries</i>
 * <p>The class is written to the given location and packageName. The location should not include the
 * package directory itself. If the package directory does not exist it will be created. If the location
 * does not exist you will get a <code>FileNotFoundException</code>
 * <p>The option <a href="#setOptimisticLock><code>setOptimisticLock</code></a> sets extra code used in the DataControl class to 
 * implement optimistic locking for this data. If you leave this to false, no locking mechanism is provided 
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.41.3 [Sep 28, 2007]
 * @see org.brains2b.sql.toclass.Sql2ClassConsole
 * @see org.brains2b.sql.meta.TableModel
 * @see org.brains2b.sql.toclass.Sql2DataC
 * 
 */
public class Sql2File extends SqlClassWriter {

	private String className;
	private ClassTemplate classTemp;
	private String[][] members;
	private List tableColumns;
	private boolean optimisticLock;
	private boolean m_trimValue;
    private boolean m_default;
    
    private String[] m_keywords= {"SYSDATE","new Timestamp(new java.util.Date().getTime())"};
	
	/**
	 * Constructor
	 * @param location File, set the location to write the file to, this location should exist before run is called
	 * on this class.
	 * @param packageName String, the packageName for the class, names should be separated by dots. The packagename
	 * is both used to set the package name of the class as to create or use the directory relative to location to write the file to.
	 * @param tm org.brains2b.sql.meta.SqlTableModel, the table model to base this dataclass on
	 */
	public Sql2File(File location, String packageName, TableModel tm) {
		super(location, packageName);
		setTableModel(tm);
		tableColumns = tm.getColumns();

	}

	/**
	 * Constructor
	 * <p>This constructor does not set the TableModel, this is however required
	 * for this class to be able to generate the file. You should set the tablemodel
	 * by calling <code>setTableModel(org.brains2b.sql.meta.SqlTableModel)</code>
	 * @param location File, set the location to write the file to, this location should exist before run is called
	 * on this class.
	 * @param packageName String, the packageName for the class, names should be separated by dots. The packagename
	 * is both used to set the package name of the class as to create or use the directory relative to location to write the file to.
	 * @see #setTableModel(SqlTableModel)
	 */
	public Sql2File(File location, String packageName) {
		super(location, packageName);
	}

	/**
	 * overwritten to retrieve the columns for local use
	 * @see org.brains2b.sql.toclass.SqlClassWriter#setTableModel(SqlTableModel)
	 */
	public void setTableModel(TableModel tableModel) {
		super.setTableModel(tableModel);
		tableColumns = tableModel.getColumns();
	}

	/**
	 * run does the actual work in this class.
	 * <p>Output is written to the <code>org.brains2b.log.Logger</code>. Dependant
	 * on the setting there the code will show up on <code>System.out</code>. This is
	 * the case if you do nothing
	 * @see java.lang.Runnable#run()
	 * @see org.brains2b.task.DefaultTask#setFinished(boolean)
	 * @see org.brains2b.log.Logger
	 */
	public void run() {
		className = getClassName(m_tableModel.getName());
		classTemp = new ClassTemplate(m_packageName, className, null, null);

		makeMembers();
		writeHeaderComment();
		writeDefaultConstructor();
		writeConstructor();
		writeAccessors();
        if (isOptimisticLock()) {
            classTemp.createMethod("serialID", "int", null).addText("return serialID;");
        }
		if (m_tableModel.getPrimaryKey()!=null) {
			writeEquals();
		}
		writeHashCode();
		writeMembers();
		try {
			classTemp.writeClass(m_location.toString());
			Logger.println("data class saved as " + className + ".java");
			setResult(RESULT_OK);
		} catch (IOException ex) {
			Logger.printException(ex);
			setResult(RESULT_ERROR);
			addMessage(ex.getMessage());
		}

		setFinished(true);
	}

	/**
	 * get the file which has been created. 
	 * <p>The result of this file is only valid after <code>run()</code>has finished
	 * @see org.brains2b.sql.toclass.SqlClassWriter#getResultfile()
	 * @see #run()
	 */
	public File getResultfile() {
		return classTemp.getCompleteFile();
	}
	
	/**
	 * creates an Array of members {their member names, local names and Java data types} from the <code>SqlColumnModel</code>s
	 * @see org.brains2b.sql.meta.SqlColumnModel
	 */
	private void makeMembers() {

		members = new String[tableColumns.size()][4];
		for (int i = 0; i < tableColumns.size(); i++) {
			ColumnModel scm = (ColumnModel) tableColumns.get(i);
            String columnName=getUnreserverdWord(scm.getName());
			members[i][0] = JavaHelper.memberName(columnName);
			members[i][1] = JavaHelper.objectName(columnName);
			if (scm.getName().equals("ROWID")) {
				members[i][2] = "String";

			} else {
				members[i][2] =
					m_typeInfo.getJavaTypeName(
						scm.getDataType(),
						scm.getColumnSize(),
						scm.getDecimalDigits());
			}
            if (scm.getDefault()==null ||  scm.getDefault().indexOf('(')!=-1 || "NULL".equals(scm.getDefault())) {
                // Don't allow for functions, ignore explicit sets to NULL
            } else if (!isKeyWord(scm.getDefault())) {
                members[i][3]= scm.getDefault() + ("long".equals(members[i][2])?"L":"double".equals(members[i][2])?"d":"");
            } else if (scm.getDefault().indexOf('(')==-1) {  
                members[i][3]=getJavaForKeyWord(scm.getDefault());
            }
		}
	}

	/**
	 * write a default comment header to the result file
	 */
	private void writeHeaderComment() {
		CommentTemplate comment = classTemp.createComment();
		comment.addText(Sql2ClassConsole.VERSIONID);
		comment.addText("Class " + className);
		comment.addText("Generated from table: " + m_tableModel.getName());
		comment.addText("Schema: " + m_tableModel.getSchema());
		//TODO get database name
		comment.addText("Database: ");
		comment.addText("");
		comment.addText("@since: " + new Date().toString());

	}

	/**
	 * write the default empty constructor
	 */
	private void writeDefaultConstructor() {
		MethodTemplate constructor =classTemp.createConstructor((String[]) null);
        if (isOptimisticLock()) {
            constructor.addText("serialID = hashCode();");
            constructor.addText("m_checksum=serialID;");
        }
	}

	/**
	 * write a constructor for this data class with all the data members (the columns in the table)
	 * as parameters
	 */
	private void writeConstructor() {
	    // don't add a constructor with more than 255 columns, Java limit
	    if (members.length>126) return;
		String[][] params = new String[2][members.length];
		for (int i = 0; i < members.length; i++) {
			params[0][i] = members[i][2];
			params[1][i] = varName(members[i][1]);
		}
		MethodTemplate constructor = classTemp.createConstructor(params);
		if (isOptimisticLock()) {
            constructor.addText("serialID = hashCode();");
        }
        for (int i = 0; i < members.length; i++) {
			constructor.addText(
				members[i][0] + " = " + varName(members[i][1]) + ";");
		}
	}
	
	/**
	 * write a getter and setter for each data member (column in the table)
	 * <p>For dates two setters are written: one to set a Timestamp or Date (dependant on SqlColumnModel.columnType)
	 * and one to set a java.util.Date
	 * @see org.brains2b.sql.meta.SqlColumnModel#getColumnType
	 * @see org.brains2b.sql.meta.Types2Java#getJavaDataType(int, int, int)
	 */
	private void writeAccessors() {
		if (isOptimisticLock()) {
			classTemp.addInterface("org.brains2b.data.OptimisticLock");
			writeGetter("m_checksum", "Checksum", "int");
			writeSetter("m_checksum", "Checksum", "int");
		}

		for (int i = 0; i < members.length; i++) {
			String method=members[i][1];
			if (KeyWords.isMethodReserved( "get"+ method)) { // check for reserved word and change both names
				method=method+"X";
			}
			//TODO Write getter and setter comments
			writeGetter(members[i][0], method, members[i][2]);
			writeSetter(members[i][0], method, members[i][2]);
			
			if ("Clob".equals(members[i][2]) || "Blob".equals(members[i][2])) {
				classTemp.addImport("java.sql." + members[i][2]);
			} else if ("Stream".equals(members[i][2])) {
				classTemp.addImport("org.brains2b.sql.meta.types.Stream");
			}
			
			if (members[i][2].equals("Timestamp")) {
				classTemp.addImport("java.sql.Timestamp");
				writeDateSetter(
					"Timestamp",
					members[i][0],
					method,
					"java.util.Date");
			}
			if (members[i][2].equals("Date")) {
				classTemp.addImport("java.sql.Date");
				writeDateSetter(
					"Date",
					members[i][0],
					method,
					"java.util.Date");
			}

		}
	}

	/**
	 * write an equals method for this class based on the primary keys of the table.
	 * <p>This ensures an equals method based on primary keys only. Otherwise the 
	 * hashCode value is used as in the default equals method.
	 * <p>This method is only called when the table this class is based on contains
	 * primary keys
	 */
	private void writeEquals() {

		ContentTemplate equ =
			classTemp
				.createMethod(
					Modifier.PUBLIC,
					"equals",
					"boolean",
					new String[][] { { "Object" }, {
				"o" }
		});
		equ.addText("boolean result=false;");
		IfTemplate ifTemp = equ.createIf("o instanceof " + className);
		ifTemp.addText(className + " l = (" + className + ") o;");
		StringBuffer sb = new StringBuffer();
		int cnt=0; // count primary keys
		for (int i = 0; i < tableColumns.size(); i++) {
			ColumnModel cm = (ColumnModel) tableColumns.get(i);
			if (isPrimaryKey(cm, m_tableModel)) {
				if (cnt > 0) {
					sb.append(" && ");
				}
				if (m_typeInfo.isPrimitive(cm.getDataType())) {
					sb.append(
						" this."
							+ members[i][0]
							+ " == l.get"
							+ members[i][1]
							+ "() ");
				} else {
					sb.append(
						" this."
							+ members[i][0]
							+ ".equals(l.get"
							+ members[i][1]
							+ "()) ");
				}
				cnt++;
			}
		}
		ifTemp.createIf(sb.toString()).addText("result = true;");
		equ.addText("return result;");

	}

	/** 
	 * write members
	 */
	private void writeMembers() {
		if (isOptimisticLock()) {
			classTemp.createMember("int", "m_checksum");
            classTemp.createMember(Modifier.PRIVATE+Modifier.FINAL,"int", "serialID",null);
		}

		for (int i = 0; i < members.length; i++) {
			Object def = null;
			if (m_default && members[i][3]!=null) {
				if ("String".equals(members[i][2])) {
					def = new Literal(members[i][3]);
				} else if ("char".equals(members[i][2])) {
					def = new Literal(members[i][3].charAt(0));
				} else {
					def = members[i][3];
				}
			}
			classTemp.createMember(members[i][2], members[i][0],def);
		}
	}

	/**
	 * write a hashCode which is based on every members value (for primitives)
	 * and hashCodes (for objects) added on odd members or substracted on even
	 * members. 
	 */
	private void writeHashCode() {
		ContentTemplate hash =
			classTemp.createMethod(
				Modifier.PUBLIC,
				"hashCode",
				"int",
				(String[]) null);
		hash.addImport("org.brains2b.data.HashCodeHelper");
		hash.addText("int hash=HashCodeHelper.INITIAL;");
		for (int i = 0; i < members.length; i++) {
			hash.addText("hash = HashCodeHelper.hash(hash, get" + members[i][1] +"());");
		}
		hash.addText("return hash;");
	}

	/**
	 * write an individual getter
	 * @param mName String , member name (m_)
	 * @param vName String, variable name, used to make the method name
	 * @param type String, representation of the Java type
	 */
	private void writeGetter(String mName, String vName, String type) {
		ContentTemplate get =
			classTemp.createMethod(
				Modifier.PUBLIC,
				"get" + vName,
				type,
				(String[]) null);
		if (m_trimValue && type.equals("String")) {
			get.createIf(mName +" != null && " + mName +".trim().length()>0").addText("return " + mName+".trim();");
			get.addText("return \" \";");
		}  else {
			get.addText("return " + mName + ";");
		}
		
	}

	/**
	 * write an individual setter
	 * @param mName String , member name (m_)
	 * @param vName String, variable name, used as parameter and to make the methodname
	 * @param type String, representation of the Java type
	 */
	private void writeSetter(String mName, String vName, String type) {
		ContentTemplate set =
			classTemp
				.createMethod(
					Modifier.PUBLIC,
					"set" + vName,
					"void",
					new String[][] { { type }, {
				varName(vName)
				}
		});
		set.addText(mName + " = " + varName(vName) + ";");
	}

	/**
	 * write an individual setter for a java.util.Date
	 * @param sqltype String, completely qualified name for either <code>java.sql.Timestamp</code>
	 * or <code>java.sql.Date</code>
	 * @param mName String , member name (m_)
	 * @param vName String, variable name
	 * @param type String, representation java.util.Date
	 * @see java.sql.Date
	 * @see java.sql.Timestamp
	 */
	private void writeDateSetter(
		String sqltype,
		String mName,
		String vName,
		String type) {
		ContentTemplate set =
			classTemp
				.createMethod(
					Modifier.PUBLIC,
					"set" + vName,
					"void",
					new String[][] { { type }, {
				varName(vName)
				}
		});
		IfTemplate ifTemp = set.createIf(varName(vName) + "!=null");
		ifTemp.addText(
			mName
				+ "= new "
				+ sqltype
				+ "(("
				+ varName(vName)
				+ ").getTime());");
		if (!type.endsWith(type)) {
			ifTemp.addText(mName +".setNanos(0);");
		}
		ifTemp.createElse().addText(mName + " = null;");

	}

	/**
	 * get the class name for the data class
     * @see org.brains2b.sql.toclass.SqlClassWriter#getClassName(String)
     */
    protected String getClassName(String name) {
        name=getUnreserverdWord(name);
        return JavaHelper.objectName(name);
    }	
    
    
    
	/**
	 * does this class use optimistic locking
	 * @return true if optimistic locking should be used, false otherwise
	 */
	protected boolean isOptimisticLock() {
		return optimisticLock;
	}

	/**
	 * set this to true if optimistic locking should be used.
	 * <p>by default optimistic locking is not used.
	 * @param b boolean
	 */
	public void setOptimisticLock(boolean b) {
		optimisticLock = b;
	}

	/**
	 * @return Returns the trimValue.
	 */
	public boolean isTrimValue() {
		return m_trimValue;
	}
	/**
	 * @param trimValue The trimValue to set.
	 */
	public void setTrimValue(boolean trimValue) {
		m_trimValue = trimValue;
	}

	/**
	 * check if defaults should be generated for the class
	 * @return true if default should be generated, false 
	 * otherwise
	 */
    public boolean isDefault() {
        return m_default;
    }

    /** set indicator to generate default values
     * from table as default in the class
     * @param default1 boolean, true if default should be
     * generated, false otherwise
     */
    public void setDefault(boolean default1) {
        m_default = default1;
    }
    
    /**
     * check if the default value is an SQL keyword
     * @param word String the default value
     * @return boolean, true if this is a keyword, false
     * otherwise
     */
    private final boolean isKeyWord(String word) {
        if (StringHelper.isEmpty(word)) return false;
        String w = word.toUpperCase();
        for (int i=0;i<m_keywords.length;i+=2) {
            if (w.equals(m_keywords[i])) return true;
        }
        return false;
    }
    
    /**
     * get the Java equivalent for the default value containing
     * an SQL keyword
     * @param word String the default value
     * @return String, the Java equivalent of this keyword
     */
    private final String getJavaForKeyWord(String word) {
        if (StringHelper.isEmpty(word)) return null;
        String w = word.toUpperCase();
        for (int i=0;i<m_keywords.length;i+=2) {
            if (w.equals(m_keywords[i])) return m_keywords[i+1];
        }
        return null;
    }
}
