/*     Sql2DataCTest.java
 *	
 *     Created on: Mar 23, 2004
 *     Created by: Dennis Groenendijk
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
package test.org.brains2b.sql.toclass;

import java.io.File;
import java.sql.Types;

import org.brains2b.sql.SQLRecord;
import org.brains2b.sql.meta.ColumnModel;
import org.brains2b.sql.toclass.Sql2DataC;
import org.brains2b.task.Task;

/** Unit test for  org.brains2b.sql.toclass.Sql2DataC
 * 
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.10 [Mar 23, 2004]
 * @see org.brains2b.sql.toclass.Sql2DataC
 */
public class Sql2DataCTest extends TestSql2Class {
	
	/*
	 * Test for void Sql2DataC(File, String)
	 */
	public void testSql2DataCFileString() {
		Sql2DataC datac=new Sql2DataC(new File(System.getProperty("java.io.tmpdir")),"sql2class.sql2datac");
		datac.setTableModel(tm);
		datac.run();
		comparefiles(new File("test/sql2datac/CourseDC01.java"),datac.getResultfile());
	}

	/*
	 * Test for void Sql2DataC(File, String, SqlTableModel)
	 */
	public void testSql2DataCFileStringSqlTableModel() {
		Sql2DataC datac=new Sql2DataC(new File(System.getProperty("java.io.tmpdir")),"sql2class.sql2datac",tm);
		datac.run();
		comparefiles(new File("test/sql2datac/CourseDC01.java"),datac.getResultfile());
	}

	public void testSetReadOnly() {
		Sql2DataC datac=new Sql2DataC(new File(System.getProperty("java.io.tmpdir")),"sql2class.sql2datac");
		datac.setTableModel(tm);
		datac.setReadOnly(true);
		datac.run();
		comparefiles(new File("test/sql2datac/CourseDC02.java"),datac.getResultfile());
	}
	
	public void testSetView() {
        SQLRecord record = new SQLRecord(3);
        record.setColumn(0, "TABLE_TYPE",String.class, "VIEW");
        record.setColumn(1, "TABLE_NAME",String.class, "COURSE_VIEW");
        record.setColumn(2, "TABLE_SCHEM" ,String.class,"DUTCH");
    
		TableModelTester view=new TableModelTester(record);
		view.setColumns(tm.getColumns());
        view.setPrimaryKey(tm.getPrimaryKey());
		Sql2DataC datac=new Sql2DataC(new File(System.getProperty("java.io.tmpdir")),"sql2class.sql2datac");
		datac.setTableModel(view);
		datac.run();
		comparefiles(new File("test/sql2datac/CourseViewDC.java"),datac.getResultfile());
	}

	public void testSetOptimisticLock() {
		Sql2DataC datac=new Sql2DataC(new File(System.getProperty("java.io.tmpdir")),"sql2class.sql2datac");
		datac.setTableModel(tm);
		datac.setOptimisticLock(true);
		datac.run();
		comparefiles(new File("test/sql2datac/CourseDC03.java"),datac.getResultfile());
	}
	
	public void testRowId() {
		tm.getPrimaryKey().getColumns().clear();
		ColumnModel rowId=createColumn(null,"DUTCH","COURSE","ROWID",Types.OTHER,0,0,null,4);
		tm.getPrimaryKey().getColumns().add(rowId);
		tm.getColumns().add(rowId);
		Sql2DataC datac=new Sql2DataC(new File(System.getProperty("java.io.tmpdir")),"sql2class.sql2datac");
		datac.setTableModel(tm);
		datac.run();
		comparefiles(new File("test/sql2datac/CourseDC04.java"),datac.getResultfile());
	}
	
	public void testDoublePK() {
		//((ColumnModel) tm.getColumns().get(1)).setPrimaryKey(2);
        ((TestSql2Class.ConstraintModelTester) tm.getPrimaryKey()).addColumn((ColumnModel)tm.getColumns().get(1)); 
		Sql2DataC datac=new Sql2DataC(new File(System.getProperty("java.io.tmpdir")),"sql2class.sql2datac");
		datac.setTableModel(tm);
		datac.run();
		comparefiles(new File("test/sql2datac/CourseDC05.java"),datac.getResultfile());
	}

	public void testNoPK() {
		tm.setPrimaryKey(null);
		Sql2DataC datac=new Sql2DataC(new File(System.getProperty("java.io.tmpdir")),"sql2class.sql2datac");
		datac.setTableModel(tm);
		datac.run();
		assertEquals(Task.RESULT_ERROR,datac.getResult());
		assertEquals("Table model has no primary key, pseudo primary key or ROWID. It is not possible to generate the DataControl",datac.getMessages().pop());
	}
    
}
