package test.org.brains2b.sql.toclass;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.sql.Types;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

import org.brains2b.sql.SQLRecord;
import org.brains2b.sql.meta.ColumnModel;
import org.brains2b.sql.meta.ConstraintModel;
import org.brains2b.sql.meta.TableModel;

public abstract class TestSql2Class extends TestCase {

	public class ConstraintModelTester extends ConstraintModel {
	    public ConstraintModelTester(String type, SQLRecord record) {
	            super(type, record);
	    }
	    public void addColumn(ColumnModel model) {
	        super.addColumn(model);
	    }
	}

	public class TableModelTester extends TableModel {
	    public TableModelTester(SQLRecord record) {
	            super(record);
	    }
	    public void setColumns(List columns) {
	        super.setColumns(columns);
	    }
	    public void setPrimaryKey(ConstraintModel primaryKey) {
	        super.setPrimaryKey(primaryKey);
	    }
	}

	protected TableModelTester tm;
	protected ConstraintModelTester pk;
	
	public void setUp() {
        tm = createTestCourse();
        pk = (ConstraintModelTester) tm.getPrimaryKey();
	}
	
	public TableModelTester createTestCourse() {
		SQLRecord record = new SQLRecord(3);
        record.setColumn(0, "TABLE_TYPE",String.class, "TABLE");
        record.setColumn(1, "TABLE_NAME",String.class, "COURSE");
        record.setColumn(2, "TABLE_SCHEM" ,String.class,"DUTCH");
        TableModelTester tm=new TableModelTester(record);
        
        ArrayList columns=new ArrayList();
        ColumnModel col1 = createColumn(null,"DUTCH","COURSE","ID",Types.DECIMAL,9,0,null,1);
        columns.add(col1);
        columns.add(createColumn(null,"DUTCH","COURSE","NAME",Types.VARCHAR,30,0,null,2));
        columns.add(createColumn(null,"DUTCH","COURSE","TEACHER",Types.DECIMAL,9,0,null,3));
        columns.add(createColumn(null,"DUTCH","COURSE","ROOM_ID",Types.DECIMAL,9,0,null,3));
        columns.add(createColumn(null,"DUTCH","COURSE","STARTDATE",Types.TIMESTAMP,9,0,null,3));
        columns.add(createColumn(null,"DUTCH","COURSE","ENDDATE",Types.TIMESTAMP,9,0,null,3));
        tm.setColumns(columns);
	
        record = new SQLRecord(2);
        record.setColumn(0, "NAME",String.class, "PK");
        record.setColumn(1, "TABLE_NAME",String.class, "COURSE");
        ConstraintModelTester pk = new ConstraintModelTester(ConstraintModel.PRIMARY_KEY, record);
        pk.addColumn(col1);
        tm.setPrimaryKey(pk);
        return tm;
	}
	
	public ColumnModel createColumn(String catalog, String schema, String tableName, String columnName,int dataType,int columnSize,int decimalDigits, String dflt, int sequence) {
        SQLRecord record = new SQLRecord(9);
        record.setColumn(0, "TYPE"      ,String.class,"COLUMN");
        record.setColumn(1, "COLUMN_NAME",String.class, columnName);
        record.setColumn(2, "TABLE_CAT"  ,String.class, catalog);
        record.setColumn(3, "TABLE_SCHEM",String.class, schema);
        record.setColumn(4, "TABLE_NAME" ,String.class, tableName);
        record.setColumn(5, "DATA_TYPE"  ,Integer.class, new Integer(dataType));
        record.setColumn(6, "COLUMN_SIZE",Integer.class, new Integer(columnSize));
        record.setColumn(7, "DECIMAL_DIGITS" ,Integer.class, new Integer(decimalDigits));
        record.setColumn(8, "ORDINAL_POSITION",Integer.class, new Integer(sequence));
        record.setColumn(9, "COLUMN_DEF"  ,String.class, dflt);
        return new ColumnModel(record);
    }

	public TableModelTester createNotes(int dataType) {
		SQLRecord record = new SQLRecord(3);
        record.setColumn(0, "TABLE_TYPE",String.class, "TABLE");
        record.setColumn(1, "TABLE_NAME",String.class, "NOTES");
        record.setColumn(2, "TABLE_SCHEM" ,String.class,"DUTCH");
        TableModelTester tm=new TableModelTester(record);
        
        ArrayList columns=new ArrayList();
        ColumnModel col1 = createColumn(null,"DUTCH","NOTES","ID",Types.DECIMAL,9,0,null,1);
        columns.add(col1);
        columns.add(createColumn(null,"DUTCH","NOTES","TEXT",dataType,0,0,null,2));
        tm.setColumns(columns);
        
        record = new SQLRecord(2);
        record.setColumn(0, "NAME",String.class, "PK");
        record.setColumn(1, "TABLE_NAME",String.class, "NOTES");
        ConstraintModelTester pk = new ConstraintModelTester(ConstraintModel.PRIMARY_KEY, record);
        pk.addColumn(col1);
        tm.setPrimaryKey(pk);
        return tm;
	}
	
	protected void comparefiles(File left, File right) {
		try {
			BufferedReader lbr=new BufferedReader(new FileReader(left));
			BufferedReader rbr=new BufferedReader(new FileReader(right));
			String lline=lbr.readLine();
			String rline=rbr.readLine();
			int count=0;
			while (lline!=null) {
				count++;
				if (rline!=null) {
					if (lline.indexOf("@since")==-1) { // ignore timestamp in since for string compare
						assertEquals("difference at line:" + count,lline,rline);
					} else {
						assertEquals(lline.indexOf("@since"),rline.indexOf("@since"));
					}
				} else {
					assertNull("difference at line:" + count + ":" + "could not find it in compared file");
				}
				lline=lbr.readLine();
				rline=rbr.readLine();
			}
			assertNull("compared file still contains data",rbr.readLine());
			lbr.close();
			rbr.close();
		} catch (Exception ex) {
			assertTrue(ex.getMessage(),false);
		}
	}
	
}
